Set-StrictMode -Version Latest

# So-called profiles, i.e. sets of settings
[UIAutomation.Mode]::Profile = [UIAutomation.Modes]::Presentation;
# default: [UIAutomation.Modes]::Presentation;
# ------------------------------------------------------


# Highlighting of controls
[UIAutomation.Preferences]::Highlight = $true;
# default: $true
# ------------------------------------------------------
[UIAutomation.Preferences]::HighlighterColor = [System.Drawing.Color]::Blue;
# default: [System.Drawing.Color]::Red
# ------------------------------------------------------
#[UIAutomation.Preferences]::HighlighterBorder = 3;
# default: 3
# ------------------------------------------------------
[UIAutomation.Preferences]::HighlightParent = $true;
# default: $true
# ------------------------------------------------------
#[UIAutomation.Preferences]::HighlighterColorParent = [System.Drawing.Color]::HotPink;
# default: [System.Drawing.Color]::HotPink
# ------------------------------------------------------
#[UIAutomation.Preferences]::HighlighterBorderParent = 5;
# default: 5
# ------------------------------------------------------


# The timeout for Get-UIAWindow, Get-UIA[ControlType], Wait- cmdlets
# No more do/while cycles or sleeps
# Adjust your code with individual timeouts (per cmdlet) or 
# use one universal timeout.
# Remember, that with overly short timeout you risk not 
# to catch a window or a control
# Unnecessarily long timeout in turn swallow your time
# if a window or a control is gone. The cmdlet will wait all
# the time that you set to.
[UIAutomation.Preferences]::Timeout = 5000;
# default: 5000
# ------------------------------------------------------
# This is a time-saving way to go through a window
# If a window or a child window is gone or never appeared,
# the test suite start working with lesser timeout,
# i.e., if your test suite should do ten clicks to the window
# that is gone, you must spend 10*5=50 seconds.
# However, if the window is gone, ten clicks will take
# only 10*2=20 seconds.
#[UIAutomation.Preferences]::AfterFailTurboTimeout = 2000;
# default: 2000
# ------------------------------------------------------


# Types of search used when we are searching for a control            
# the classical UIAutomaiton's FindFirst query
#[UIAutomation.Preferences]::DisableExactSearch = $true;
# default: $true
# ------------------------------------------------------
# the FindWindowEx search. The search supports wildcards.
#[UIAutomation.Preferences]::DisableWildCardSearch = $false;
# default: $false
# ------------------------------------------------------
#[UIAutomation.Preferences]::DisableWin32Search = $false;
# default: $false
# ------------------------------------------------------


# There is also the possibility to take screenshots automatically
# if a cmdlet fails            
#[UIAutomation.Preferences]::ScreenShotFolder = 
# default: user's TEMP folder
# ------------------------------------------------------
#[UIAutomation.Preferences]::OnErrorScreenShot = $false;
# default: $false
# ------------------------------------------------------


# The transcript interval is used in the Start-UIARecorder cmdlet
# to set the frequency the cmdlet queries the cursor position
#[UIAutomation.Preferences]::TranscriptInterval = 200;
# default: 200
# ------------------------------------------------------
# By default, you are working in the Presentation mode
# This means that after catching a control successfully
# a Get-UIAWindow, Get-UIA[ControlType] and
# several other cmdlets sleep for the OnSuccessDelay time.
# It's useful in case you need to view what's going on.
[UIAutomation.Preferences]::OnSuccessDelay = 0;
# default: 0 # 500
# ------------------------------------------------------
# You can attach one or more scriptblock to a cmdlet individually.
# In addition, you may also set one or more scriptblock to
# run after every successful cmdlet
#[UIAutomation.Preferences]::OnSuccessAction = $null;
# default: $null
# example: [UIAutomation.Preferences]::OnSuccessAction = {Start-Process calc;},{"It works!" >> C:\1\works.txt;},{sleep -Seconds 2; Save-UIAScreenshot -Description "Two calcs";}
# ------------------------------------------------------
# The time a cmdlet holds the pipeline after an error caused
#[UIAutomation.Preferences]::OnErrorDelay = 500;
# default: 500
# ------------------------------------------------------
# The default error handler(s)
#[UIAutomation.Preferences]::OnErrorAction = $null;
# default: $null
# ------------------------------------------------------
# The Get- cmdlets query Automaiton tree until the timeout expires
# However, it's not a requirement to perform queries without a sleep
# This delay is a delay betwwen subsequent queries to the Automaiton tree
#[UIAutomation.Preferences]::OnSleepDelay = 500;
# default: 500
# ------------------------------------------------------
# There is also the possibility (rarely used possibility)
# to run scriptblock(s) between performing queries.
# This may be useful if you need, for example,
# re-run a process of your AUT
#[UIAutomation.Preferences]::OnSleepAction = $null;
# default: $null
# ------------------------------------------------------
# This delay is for watching what's happened afer
# a Win32 click was performed
#[UIAutomation.Preferences]::OnClickDelay = 0;
# default: 0
# ------------------------------------------------------


# The log file is being written to the UIAutomaiton.log file in user's TEMP folder
#[UIAutomation.Preferences]::Log = $true;
# default: $true
# ------------------------------------------------------
#[UIAutomation.Preferences]::LogPath = 
# default: user's TEMP folder
# ------------------------------------------------------


# The UIAutomaiton module stores error records in the collection
# [UIAutomation.CurrentData]::Error
# that is a subset of PowerShell's $Error collections
#[UIAutomation.Preferences]::MaximumErrorCount = 256;
# ------------------------------------------------------


# The events that were subscribed to are also stores
# in a collection
# Theoretically, you can learn which events are caught
# default: 
#[UIAutomation.Preferences]::MaximumEventCount = 256;
# ------------------------------------------------------


#            // CacheRequest
#[UIAutomation.Preferences]::FromCache = $false;
# default: $false
# ------------------------------------------------------
            
#            // Test Case Management
[UIAutomation.Preferences]::EveryCmdletAsTestResult = $true;
# default: $false
# ------------------------------------------------------

[TMX.TestData]::ResetData();

#$ErrorActionPreference = [System.Management.Automation.ActionPreference]::Continue;
$ErrorActionPreference = [System.Management.Automation.ActionPreference]::SilentlyContinue;
# SIG # Begin signature block
# MIIQPAYJKoZIhvcNAQcCoIIQLTCCECkCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUBm6h3fKscqN8vyibOtcRktbZ
# 2gGgggvbMIIDPjCCAiqgAwIBAgIQTID00uKaEIdNkcM87VMYyzAJBgUrDgMCHQUA
# MC0xKzApBgNVBAMTIlNvZnR3YXJlVGVzdGluZ1VzaW5nUG93ZXJTaGVsbC5jb20w
# HhcNMTIwNDI3MjAwNzE5WhcNMzkxMjMxMjM1OTU5WjAtMSswKQYDVQQDEyJTb2Z0
# d2FyZVRlc3RpbmdVc2luZ1Bvd2VyU2hlbGwuY29tMIIBIjANBgkqhkiG9w0BAQEF
# AAOCAQ8AMIIBCgKCAQEAlcqLEHl4h/KAtVa9CEyaepXPOR+wWI8/qvO2ro2b+QH3
# SHCyLTLDyJ96auN57WA6hiEN0kPUaeE3ncOMSTR1ke6woSGtk9LOCbts4qykPQDc
# +rnXOvX46LPGEd0doI+eTOEEKQQ4RoNiTMG6Br6MhQC4G/e80viLG7oSKPPqHnXe
# XckwUaH9kgP7J9GE6hJwW13LyCyVJJtKmQB77iHRCbmCgFr+mZz7tppP5D/UBk/7
# I7h3OhnbxH9JvG8Kps0Uz/LspjeuTzkN93Y1h+eHBwIRSAz4k4i2k0MQiC6eemdz
# k3rhYFqwCzYGfoCued+m8Oo1zxjk2IE8X1OELPC9uQIDAQABo2IwYDBeBgNVHQEE
# VzBVgBAacCaZ0LNMRd6b1x9uaKmKoS8wLTErMCkGA1UEAxMiU29mdHdhcmVUZXN0
# aW5nVXNpbmdQb3dlclNoZWxsLmNvbYIQTID00uKaEIdNkcM87VMYyzAJBgUrDgMC
# HQUAA4IBAQCAr2Ck5BLGyImOX9IbZtJlM1PrwGmAhdZ9RHLIyRjhwzR5VO899C0p
# IYp86lRN2s4GolwNJQKWwyTtMakQaUJbDZcCyngwVMPG4ru592TCWKxsFfzNhQCI
# IugvagQIykDwKHO7uhq02rrjtgl8LyRAn3n5zAUsAfCj+W4zlV0YTpLFc88jCVTy
# kIzq7pD4K+Gp9+nQWbtRcnpjNbyKoRdWP/GO/Ad5Q3IXBMNbq1R6CoPepuA5epDH
# FZ0iUq0aKcnnyjAzJz76oIt3kRe7lBd/qbqy0LSL5nzQZYNWYCnsZ73wXIP0k+6X
# 7wovKXMnDJ4vRHY77fgDL1Yfd6z7ye0SMIID7jCCA1egAwIBAgIQfpPr+3zGTlnq
# S5p31Ab8OzANBgkqhkiG9w0BAQUFADCBizELMAkGA1UEBhMCWkExFTATBgNVBAgT
# DFdlc3Rlcm4gQ2FwZTEUMBIGA1UEBxMLRHVyYmFudmlsbGUxDzANBgNVBAoTBlRo
# YXd0ZTEdMBsGA1UECxMUVGhhd3RlIENlcnRpZmljYXRpb24xHzAdBgNVBAMTFlRo
# YXd0ZSBUaW1lc3RhbXBpbmcgQ0EwHhcNMTIxMjIxMDAwMDAwWhcNMjAxMjMwMjM1
# OTU5WjBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRp
# b24xMDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0Eg
# LSBHMjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALGss0lUS5ccEgrY
# JXmRIlcqb9y4JsRDc2vCvy5QWvsUwnaOQwElQ7Sh4kX06Ld7w3TMIte0lAAC903t
# v7S3RCRrzV9FO9FEzkMScxeCi2m0K8uZHqxyGyZNcR+xMd37UWECU6aq9UksBXhF
# pS+JzueZ5/6M4lc/PcaS3Er4ezPkeQr78HWIQZz/xQNRmarXbJ+TaYdlKYOFwmAU
# xMjJOxTawIHwHw103pIiq8r3+3R8J+b3Sht/p8OeLa6K6qbmqicWfWH3mHERvOJQ
# oUvlXfrlDqcsn6plINPYlujIfKVOSET/GeJEB5IL12iEgF1qeGRFzWBGflTBE3zF
# efHJwXECAwEAAaOB+jCB9zAdBgNVHQ4EFgQUX5r1blzMzHSa1N197z/b7EyALt0w
# MgYIKwYBBQUHAQEEJjAkMCIGCCsGAQUFBzABhhZodHRwOi8vb2NzcC50aGF3dGUu
# Y29tMBIGA1UdEwEB/wQIMAYBAf8CAQAwPwYDVR0fBDgwNjA0oDKgMIYuaHR0cDov
# L2NybC50aGF3dGUuY29tL1RoYXd0ZVRpbWVzdGFtcGluZ0NBLmNybDATBgNVHSUE
# DDAKBggrBgEFBQcDCDAOBgNVHQ8BAf8EBAMCAQYwKAYDVR0RBCEwH6QdMBsxGTAX
# BgNVBAMTEFRpbWVTdGFtcC0yMDQ4LTEwDQYJKoZIhvcNAQEFBQADgYEAAwmbj3nv
# f1kwqu9otfrjCR27T4IGXTdfplKfFo3qHJIJRG71betYfDDo+WmNI3MLEm9Hqa45
# EfgqsZuwGsOO61mWAK3ODE2y0DGmCFwqevzieh1XTKhlGOl5QGIllm7HxzdqgyEI
# jkHq3dlXPx13SYcqFgZepjhqIhKjURmDfrYwggSjMIIDi6ADAgECAhAOz/Q4yP6/
# NW4E2GqYGxpQMA0GCSqGSIb3DQEBBQUAMF4xCzAJBgNVBAYTAlVTMR0wGwYDVQQK
# ExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjEwMC4GA1UEAxMnU3ltYW50ZWMgVGltZSBT
# dGFtcGluZyBTZXJ2aWNlcyBDQSAtIEcyMB4XDTEyMTAxODAwMDAwMFoXDTIwMTIy
# OTIzNTk1OVowYjELMAkGA1UEBhMCVVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBv
# cmF0aW9uMTQwMgYDVQQDEytTeW1hbnRlYyBUaW1lIFN0YW1waW5nIFNlcnZpY2Vz
# IFNpZ25lciAtIEc0MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAomML
# OUS4uyOnREm7Dv+h8GEKU5OwmNutLA9KxW7/hjxTVQ8VzgQ/K/2plpbZvmF5C1vJ
# TIZ25eBDSyKV7sIrQ8Gf2Gi0jkBP7oU4uRHFI/JkWPAVMm9OV6GuiKQC1yoezUvh
# 3WPVF4kyW7BemVqonShQDhfultthO0VRHc8SVguSR/yrrvZmPUescHLnkudfzRC5
# xINklBm9JYDh6NIipdC6Anqhd5NbZcPuF3S8QYYq3AhMjJKMkS2ed0QfaNaodHfb
# Dlsyi1aLM73ZY8hJnTrFxeozC9Lxoxv0i77Zs1eLO94Ep3oisiSuLsdwxb5OgyYI
# +wu9qU+ZCOEQKHKqzQIDAQABo4IBVzCCAVMwDAYDVR0TAQH/BAIwADAWBgNVHSUB
# Af8EDDAKBggrBgEFBQcDCDAOBgNVHQ8BAf8EBAMCB4AwcwYIKwYBBQUHAQEEZzBl
# MCoGCCsGAQUFBzABhh5odHRwOi8vdHMtb2NzcC53cy5zeW1hbnRlYy5jb20wNwYI
# KwYBBQUHMAKGK2h0dHA6Ly90cy1haWEud3Muc3ltYW50ZWMuY29tL3Rzcy1jYS1n
# Mi5jZXIwPAYDVR0fBDUwMzAxoC+gLYYraHR0cDovL3RzLWNybC53cy5zeW1hbnRl
# Yy5jb20vdHNzLWNhLWcyLmNybDAoBgNVHREEITAfpB0wGzEZMBcGA1UEAxMQVGlt
# ZVN0YW1wLTIwNDgtMjAdBgNVHQ4EFgQURsZpow5KFB7VTNpSYxc/Xja8DeYwHwYD
# VR0jBBgwFoAUX5r1blzMzHSa1N197z/b7EyALt0wDQYJKoZIhvcNAQEFBQADggEB
# AHg7tJEqAEzwj2IwN3ijhCcHbxiy3iXcoNSUA6qGTiWfmkADHN3O43nLIWgG2rYy
# tG2/9CwmYzPkSWRtDebDZw73BaQ1bHyJFsbpst+y6d0gxnEPzZV03LZc3r03H0N4
# 5ni1zSgEIKOq8UvEiCmRDoDREfzdXHZuT14ORUZBbg2w6jiasTraCXEQ/Bx5tIB7
# rGn0/Zy2DBYr8X9bCT2bW+IWyhOBbQAuOA2oKY8s4bL0WqkBrxWcLC9JG9siu8P+
# eJRRw4axgohd8D20UaF5Mysue7ncIAkTcetqGVvP6KUwVyyJST+5z3/Jvz4iaGNT
# mr1pdKzFHTx/kuDDvBzYBHUxggPLMIIDxwIBATBBMC0xKzApBgNVBAMTIlNvZnR3
# YXJlVGVzdGluZ1VzaW5nUG93ZXJTaGVsbC5jb20CEEyA9NLimhCHTZHDPO1TGMsw
# CQYFKw4DAhoFAKBSMBAGCisGAQQBgjcCAQwxAjAAMBkGCSqGSIb3DQEJAzEMBgor
# BgEEAYI3AgEEMCMGCSqGSIb3DQEJBDEWBBThc5SOHXmwmPmW6xO5Q8/sjvjt6DAN
# BgkqhkiG9w0BAQEFAASCAQBmctKCf/2TYcO274ZLrzlRJI0Rer+a1GfBvRglGXqd
# e3tpt8GZdrQW5iFARrVdRsB16WRPHB2YN1cRZKBY+HxNXUYe68ur2hSYK+3S9OK+
# oBkQ775NmTXcy8UEWsrCvvhrO7aQitHsmVf9GCz1pDe9tSUSkyTJxEMnjqSdwpXt
# wcULMfhO3c7r2iwSs+XohY0lCuSkY+tAL7FGf3v6RO9n5F4PtNCxa5bwMghS8axM
# dYvMCS4CeiTKhZkvcF9RxswEafJpbr8phko+4BfgWYtbLpR7EFrVzy/Ozx3TRzB6
# thMIsaj8MdRI6Fp5A4p/TBcg9VctmK+XIlwB/DZRz72DoYICCzCCAgcGCSqGSIb3
# DQEJBjGCAfgwggH0AgEBMHIwXjELMAkGA1UEBhMCVVMxHTAbBgNVBAoTFFN5bWFu
# dGVjIENvcnBvcmF0aW9uMTAwLgYDVQQDEydTeW1hbnRlYyBUaW1lIFN0YW1waW5n
# IFNlcnZpY2VzIENBIC0gRzICEA7P9DjI/r81bgTYapgbGlAwCQYFKw4DAhoFAKBd
# MBgGCSqGSIb3DQEJAzELBgkqhkiG9w0BBwEwHAYJKoZIhvcNAQkFMQ8XDTE0MDIy
# NDA3MjYzN1owIwYJKoZIhvcNAQkEMRYEFH9mHbKBEQrdTCau83L8HuBoFe/7MA0G
# CSqGSIb3DQEBAQUABIIBAFTFSj/jcqMDd7/aghp3WyTRaLL7s9bE9OaykakvkBKS
# HFcgpJqauJh7hhLAW4Y55SUdMuwRnktAjVpLlCydDUuFCgVLWjZCE8eH3S+8zEPh
# hgiBg7qJuXVZqQpNQFZm5uzVJC5nM9qwAmvPHPwC/mSe7erK8XThbrTe78EiA3EW
# wjPiBEZAOwRaeqD1EGaSo3wytCusUTEa+jVYPPLhE2O03BKACDtqMDY1lMPPlsJ6
# iCSsVhPLv6KBM8P1QJdOZ2nuaYk/u2lXOI82x3KvFgUTtNQgCCG+zm21bLmVLnkS
# EdtVOufxe18MUxwPdb+nMcQxj6NGq0H2h1/lPYKh0pA=
# SIG # End signature block
